import React, { useState, useEffect, useRef, useCallback } from 'react';
import { getAvailability, setAvailability } from './api';


// adapted from https://stackoverflow.com/a/60316951/13917490
const START_DATE = new Date('05/29/2022');
const END_DATE = new Date('09/03/2022');
const DATES = Array.from(
  { length: ((END_DATE - START_DATE) / 864e5) + 1 },
  (_, i) => {
    const date = new Date(START_DATE);
    date.setDate(date.getDate()+i);
    return { month: date.getMonth() + 1, day: date.getDate() };
  }
);

const NameForm = ({ onNameSet }) => {
  const [name, setName] = useState("");

  return (
    <div className="login-form">
      <span>Vacation Planner</span>
      <span>Enter the name you want to use for the planner</span>
      <span>Remember the name you use b/c you'll need it to log back in at a later time</span>
      <span>Please just behave and use a reasonable name</span>
      <span>I spend my day job cleaning up edge cases from bad users</span>
      <form
        className="login-form__name-form"
        onSubmit={() => onNameSet(name.trim())}
      >
        <input
          className="login-form__textbox"
          type="text"
          onChange={({ target }) => setName(target.value)}
          value={name}
        />
        <input
          className={`login-form__submit ${name !== "" ? "login-form__submit--enabled" : ""}`}
          disabled={name === ""}
          type="submit"
          value="Submit"
        />
      </form>
    </div>
  )
}

const getNext = (user, availability) => ({
  yes: "no",
  no: "maybe",
  maybe: "unknown",
  unknown: "yes",
})[availability.find(({ name }) => name === user)?.status ?? "unknown"];

const pad = num => {
  let snum = "" + num
  if (snum.length === 1) snum = "0" + snum
  return snum
}

const Tile = ({ user, month, day, availability, showOthers, onClick }) => (
  <div
    onClick={() => onClick(month, day, getNext(user, availability))}
    className={`week__tile week__tile--${availability.filter(({ name }) => name === user)[0]?.status ?? "unknown"}`}
  >
    <span>{pad(month)}/{pad(day)}</span>
    {
      showOthers && availability
        .filter(({ status }) => status && status !== "unknown")
        .filter(({ name }) => name !== user)
        .map(({ name, status }) => (
          <span
            key={name}
            className={`week__availability week__availability--${status ?? "unknown"}`}
          >
            {name}
          </span>
        ))
      }
  </div>
);


const chunk = (arr, len) => {
  const chunks = [];
  for (let i = 0; i < arr.length;) {
    chunks.push(arr.slice(i, i += len));
  }
  return chunks;
}

const DaysOfWeek = () => (
  <div className="header__day-row">
    {
      ["Sun", "Mon", "Tue", "Wed", "Thr", "Fri", "Sat"]
      .map(day => (
        <div
          key={day}
          className="header__day"
        >
          {day}
        </div>
      ))
    }
  </div>
)

const Header = ({ name }) => (
  <>
    <span className="header__item">Ignore how ugly this site is I wrote it in a total of 15 hours</span>
    <span className="header__item">Logged in as: {name}</span>
    <span className="header__item">Please toggle the tiles as follows:</span>
    <div className="header__item header__list">
      <span><span className="header__green-text">Green</span> if you are available/can probably be available with enough forewarning</span>
      <span><span className="header__red-text">Red</span> if you are definitely not available</span>
      <span><span className="header__yellow-text">Yellow</span> if you don't think you will be available but can't be sure this far out</span>
    </div>
    <span className="header__item">If you are leaving a tile blank I will assume you truly have no idea</span>
    <span className="header__item">W buttons toggle a whole week</span>
  </>
)

const Week = ({ days, user, change, showOthers }) => (
  <div className="week">
    <div className="week__days">
      {
        days.map(item => (
          <Tile
            key={item.day}
            user={user}
            month={item.month}
            day={item.day}
            availability={item.availability}
            showOthers={showOthers}
            onClick={(month, day, status) => change(user, [{ month, day, status }])}
          />
        ))
      }
    </div>
    <button
      onClick={() => change(user, days.map(({ month, day, availability }) => ({
        month,
        day,
        status: getNext(user, availability),
      })))}
      className="week__button"
    >
      W
    </button>
  </div>
)

const useCalendar = () => {
  const [calendar, setCalendar] = useState(null);
  const lastUpdatedRef = useRef(null);
  const updateCalendar = useCallback(async () => {
    const { availability, lastUpdated } = await getAvailability();
    if (lastUpdated !== lastUpdatedRef.current) {
      // only actually update if there are changes
      lastUpdatedRef.current = lastUpdated;
      const newCalendar = new Map();
      availability.forEach(({ month, day, availability: availabilityList }) => {
        newCalendar.set(`${month}-${day}`, availabilityList);
      });
      setCalendar(newCalendar);
    }
  }, [lastUpdatedRef]);

  useEffect(() => {
    // do the first update
    updateCalendar();
    // update every few seconds
    const interval = setInterval(updateCalendar, 5000);
    return () => { clearInterval(interval); };
  }, [updateCalendar]);

  const getter = (month, day) => calendar?.get(`${month}-${day}`) ?? [];

  const change = useCallback(async (name, updates) => {
    await setAvailability(name, updates);
    await updateCalendar();
  }, [updateCalendar]);

  return [getter, change];
}

function App() {
  const [name, setName] = useState(null);
  const [getDay, change] = useCalendar();
  const [showAvailability, setShowAvailability] = useState(false);

  if (name === null) {
    return <NameForm onNameSet={setName} />;
  }

  return (
    <div className="page">
      <Header name={name} />
      <button 
        className="toggle-availability"
        onClick={() => setShowAvailability(s => !s)}
      >
        {showAvailability ? "Hide": "Show"} Other Responses
      </button>
      <DaysOfWeek />
      {
        chunk(DATES, 7).map(days =>
          <Week
            key={`${days[0].month}-${days[0].day}`}
            days={days.map(({ month, day }) => ({ month, day, availability: getDay(month, day) }))}
            change={change}
            user={name}
            showOthers={showAvailability}
          />
        )
      }
    </div>
  );
}

export default App;
